# coding=utf-8

# Користувач:BotCat/Вільні ліцензії без джерела
self_licenses = r'''
PD\-[Ss]elf(\s*\|.*?)?
GFDL\-[Ss]elf(\s*\|.*?)?
[Ss]elf(2)?\s*\|\s*(GFDL|PD|PU|CC(\-BY)?(\-SA)?(\-\d\.\d)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?|cc\-by\-sa\-2\.5\,2\.0\,1\.0)
[Ss]elf(2)\s*\|\s*(GFDL|PD|PU|CC(\-BY)?(\-SA)?(\-\d\.\d)?(\,2\.0\,1\.0)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?)\s*\|\s*(GFDL|PD|PU|CC(\-BY)?(\-SA)?(\-\d\.\d)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?(\,2\.0\,1\.0)?)
[Ss]elf(2)\s*\|\s*(GFDL|PD|PU|CC(\-BY)?(\-SA)?(\-\d\.\d)?(\,2\.0\,1\.0)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?)\s*\|\s*(GFDL|PD|PU|CC(\-BY)?(\-SA)?(\-\d\.\d)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?(\,2\.0\,1\.0)?)
[Ss]elf(2)\|\GFDL\|\cc-by-sa-3.0,2.5,2.0,1.0
[Ss]elf(2)?\s*\|\s*(GFDL|CC(\-BY)?(\-SA)?(\-\d\.\d)?|[Cc][Cc](\-by)?(\-sa)?(\-\d\.\d)?|CC\-BY\-SA\-(3\.0\,)?2\.5\,2\.0\,1\.0|[Cc]c\-by\-sa\-(3\.0\,)?2\.5\,2\.0\,1\.0)\s*\|\s*(GFDL|CC(\-BY)?(\-SA)?(\-\d\.\d)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?|CC\-BY\-SA\-(3\.0\,)?2\.5\,2\.0\,1\.0|[Cc]c\-by\-sa\-(3\.0\,)?2\.5\,2\.0\,1\.0)
PD\-because\s*\|.+?
PD\-user\s*\|.+?
PD\-user1\s*\|.+?
PD\-user\-(w|en|de|he)\s*\|.+?
PD\-(ineligible|trivial)
PD\-textlogo
[Pp][Dd]-[Oo]ld
PD\-signature
'''

# Користувач:BotCat/Вільні ліцензії з джерелом
free_licenses = r'''
GFDL
GFDL\s*\|.+?
GFDL\-user\s*\|.+?
GFDL\-user\-(w|en|de|ja|ru)\s*\|.+?
GFDL\-OpenGeoDB
GNU[ _]FDL

PD\-([Oo]ld|[Aa]rt)
PDart
PD\-author\s*\|.+?
PD\-(Russia|Ukraine|Sweden|Argentina)
PD\-(RU|UA|AM|BY)\-exempt
PD\-Old\-Russia(1953)?
PD\-кадр
PD\-герб
PD\-Russia
PD\-UA
PD\-BY
PD\-India
PD\-BritishGov
[Гг]ербУА

PD\-Ital(y|ia)
PD\-Japan
PD\-USGov(\-CIA\-WF|\-CIA\-WF\-RU|\-NASA|\-NOAA|\-DOE|\-POTUS)?
PD\-US(\-JPL|\-patent|\-flag)?
[Pp]D\-USGov\-(Interior\-USGS|Military\-Badge)
PD\-GWPDA
PD\-NASA
[Pp]D\-USGov\-DOJ
Патент[ _]США
PD\-Argentina

GPL
LGPL
FAL
CC\-BY(\-SA)?(\-\d\.\d)?
CC\-SA(\-BY)?(\-\d\.\d)?
CC\-BY\-SA\-3\.0\,2\.5\,2\.0\,1\.0
CC\-by\-2\.0
[Cc][Cc]\-by\-3\.0
[Cc]c\-by\-2\.[05]
[Cc]c\-by\-sa\-3\.0\,2\.5\,2\.0\,1\.0
[Cc]c\-sa\-1\.0
[Cc]c\-by\-sa\-2\.[05]
[Cc]c\-by\-sa
[Cc]c\-by\-2\.0
[Сc]с\-by\-sa\-2\.5\,2\.0\,1\.0
[Cc][Cc]\-by\-sa\-3\.0
[Cc]c\-by\-nc\-sa\-2\.0\-dual
[Cc]c\-by\-sa\-2\.1\-jp
[Cc][Cc]\-[Zz]ero

[Kk]remlin\.ru\-image
[Ff]ree[ _]?use.*?
[Uu]se[ _]Free
[Ff]ree[ _]use[ _]provided[ _]that\s*\|.+?
[Cc]opyrightedFreeUseProvidedThat\s*\|.+?
[Cc]opyrightedFreeUse
[Aa]ttribution
[Nn]oRightsReserved

[Бб]ілецький
[Вв]ласенко
'''

# Користувач:BotCat/Невільні ліцензії без джерела
unfree_licenses = r'''
[Гг]азета
[Гг]ерби?
[Гг]роші
[Ее]мблема
[Зз]німок([ _](екран[ау]|сайту|[Вв]ікі(педії)?|програми|гри))?
[Зз]німок[ _]екран[ау][ _]сайту
[Кк]адр
[Кк]адр2.*?
[Лл]ого(тип)?
[Оо]бкладинка
[Оо]бкладинка[ _]альбому
[Оо]бкладинка[ _](фільму|dvd|DVD|відеокасети|комп'ютерної[ _]гри|гри)
[Оо]бкладинка[ _](книги|журналу|журналу[ _]Time)
[Оо]лімпійська медаль
[Пп]лакат
[Пп]остер2.*?
[Пп]остер
[Пп]оп[ _\-]музика
[Бб]люз:Аудіо
[Пп]рапори?
[Пп]оштова[ _]марка
[Пп]рототип
[Сс]криншот
[Тт]итульний[ _]аркуш
[Фф]отопортрет[ _]мерця
[Фф]отопортрет[ _]померлого
[Фф]П

([Aa]lbum|[Dd]vdtape|DVD|[Ff]ilm|[Gg]ame)cover
([Aa]lbum|[Dd]vdtape|DVD|[Ff]ilm|[Gg]ame)[ _]cover
([Bb]ook|[Mm]agazine|[Tt]ime[ _]magazine)[ _]?cover
[Cc]oatofarms
([Ff]ilm|[Gg]ame|[Ss]oftware|[Tt]v)\-screenshot
[Ff]lag
[Ii]con
[Ll]ogo
[Mm]ac\-software\-screenshot
[Mm]oney
[Nn]on\-free[_ ]3D[_ ]art
[Nn]on\-free[_ ]album[_ ]cover
[Nn]on\-free[_ ]logo
[Nn]on\-free[ _]poster
[Nn]on\-free[ _]historic[ _]image
[Nn]on\-free[ _]computer[ _]icon
[Nn]on\-free[ _]software[ _]screenshot
[Nn]on\-free[_ ]currency
[Oo]lympic[ _]medal
[Pp]oster
[Ss]hot
[Ss]creenshot([ _]po)?
[Ss]oftwarecover
[Ss]tamp
[Tt]v\-program\-logo
[Vv]ideo(cover|pcgame)?
[Ww]ikipedia\-screenshot
[Ww]ebpromo


[Cc]opyrightByWikimedia
'''

# Користувач:BotCat/Невільні ліцензії з джерелом
unfree_licenses_source = '''
[Ff]air[ _]?use[ _]?in\s*\|.+?
[Aa]rt
[Рр]исунок
[Фф]отографія[ _]скульптури
[Aa]rt[ _]sculpture
[Cc]opyrightedFreeUseProvidedThat\s*\|.+?
[Nn]on\-free[_ ]promotional
[Nn]on\-free[_ ]Olympics[_ ]logo

[Cc]haracter\-artwork

[Іі]сторичне[ _]фото
[Hh]istorical

[Кк]арта

[Сc]кульптура
[Nn]on\-free[_ ]3D[_ ]art
[Nn]on\-free[_ ]audio[_ ]sample

[Mm]usic[ _]sample
[Сс]емпл

[Рр]ок[ _]музика\s*\|.+?
[Мм]узика\s*\|.+?
[Пп]оп\-музика\s*\|.+?


[Ss]heet[ _]music
[Нн]отний[ _]приклад
'''

# Користувач:BotCat/Застарілі ліцензії
old_licenses = '''
[Ff]air[ _]?use
[Ff]air[ _]?old
[Pp]romo[Pp]hoto
[Pp]romotional
[Pp]romotional[ _]\-[ _]товар
PD
PU
GFDL\-en
[Ff]airuse[ _]\-[ _]архітектура
[Ff]airuse[ _]\-[ _]портрет
[Ff]airuse[ _]\-[ _]предмет
[Ff]airuse[ _]\-[ _]схема
'''


# Користувач:BotCat/Заборонені ліцензії
prohibited_licenses = '''
[Зз]аборонена[ _]ліцензія
CC[ _]bad
[Nn]oncommercial
CC.*?\-NC.*?
CC.*?\-ND.*?
[Cc]c.*?\-nc.*?
[Cc]c.*?\-nd.*?
[Ss]oviet\-?PU
PD[\- _]?[Ss]oviet
[Ss]oviet[ _](pd|PD)
PD\-USSR
[Ff]airuse[ _]\-[ _]карта
[Ww]ith[ _]permission
[Pp]ermission
PD\-meteor(ite)?
'''


# Користувач:BotCat/Попередження
warnings = '''
[Dd]isputed.*?
NCT.*?
RCTC.*?
[Зз]аміна[ _]з[ _]Вікісховища.*?
[Nn]ow[ _]Commons.*?
[Vv]fd.*?
[Dd]elete
[Qq]\-delete
[Dd]b\-\w+?.*?
[Nn]o[ _]source.*?
[Nn]o[ _]license.*?
[Nn]ostatus.*?
[Бб]ез[ _]ліцензії.*?
[Бб]ез[ _]джерела.*?
'''

# Користувач:BotCat/Без шаблона
no_template = '''
GFDL
GFDL\-self
GNU[ _]FDL
GNU[ _]Free[ _]Documentation[ _]License
GPL
FDL
PD\-user
PD\-self
PD\-US
PD
PU
[Сс]успільне[ _]надбання
PD\-Art
PD\-USGov
PD\-USGov\-CIA\-WF
PD\-USGov\-NARA
PD\-USGov\-NASA
LGPL
FAL
CC(\-BY)?(\-SA)?
[Kk]remlin.ru\-image
[Ff]ree[ _]?use
PD\-Poland
fair[ _]?use
[Aa]rt
[Bb]ook[ _]cover
[Оо]бкладинка[ _]книги
[Тт]итульний[ _]аркуш
[Mm]agazine[ _]cover
[Aa]lbumcover
[Dd]vdtapecover
[Оо]бкладинка[ _]dvd
[Vv]ideopcgame
[Ll]ogo
[Лл]оготип
[Кк]адр
[Сс]кр[іи]ншот
[Зз]німое[ _]екран[ау] 
[Ss]creenshot
DVD[ _]Cover
[Pp]oster
[Pp]olitposter
[Пп]остер
[Гг]ерби?
[Пп]рапори?
[Іі]сторичний[ _]плакат
[Mm]oney
[Ss]tamp
Creative[ _]Commons
Attribution
Share\-alike
ShareAlike
'''
